import ast
import csv
import re
import tkinter as tk
from enum import Enum

import cv2
import numpy as np
from PIL import Image, ImageTk


class CheckBox_S1:
    def __init__(self, canvas, checkbox_btn_name, polygon: list, checkbox_on_off_image_path: list,
                 checkbox_centroid: tuple,
                 default_state_is_on=False, x_offset_correction=0, y_offset_correction=0, command=None):

        self.__command = command
        self.__canvas = canvas
        self.__polygon = polygon

        self.__x_offset_correction = x_offset_correction
        self.__y_offset_correction = y_offset_correction

        self.__checkbox_centroid = checkbox_centroid

        self.__tag = checkbox_btn_name
        self.__clicked_checkbox = None

        self.__default_state_is_on = default_state_is_on
        self.__x_offset_correction = x_offset_correction

        self.__selected_image = Image.open(checkbox_on_off_image_path[0])
        self.__unselected_image = Image.open(checkbox_on_off_image_path[1])

        self.__image_tag_selected = 'checkbox_img_tag_selected_' + self.__tag
        self.__image_tag_unselected = 'checkbox_img_tag_unselected_' + self.__tag

        # self.on_off_image_width, self.on_off_image_height = self.on_off_image.size
        self.__selected_photo = ImageTk.PhotoImage(self.__selected_image)
        self.__unselected_photo = ImageTk.PhotoImage(self.__unselected_image)

        __checkbox_centroid_x, __checkbox_centroid_y = checkbox_centroid[0], checkbox_centroid[1]

        self.__canvas.create_image(__checkbox_centroid_x, __checkbox_centroid_y,
                                   anchor=tk.CENTER, image=self.__selected_photo, tags=self.__image_tag_selected)
        self.__canvas.create_image(__checkbox_centroid_x, __checkbox_centroid_y,
                                   anchor=tk.CENTER, image=self.__unselected_photo, tags=self.__image_tag_unselected)

        self.__draw_polygon()
        self.__last_status = not default_state_is_on
        self.__toggle_checkbox()


    def set(self, state: bool):
        state=not state # toggle_checkbox function toggles the input so 'state' is reversed here

        if self.__last_status == state:
            self.__last_status = state
            self.__toggle_checkbox()


    @property
    def Value(self):
        return True if self.__last_status else False


    def __checkbox_mouse_dn(self, event):
        # As thie mouse down is only triggered when it's clicked in one of the two button positions so
        # no need to check whether the click has occurred within the polygon, just toggle is required
        x, y = event.x, event.y

        if self.__is_point_in_polygon(x, y, self.__polygon):
            closesttag = self.__canvas.find_closest(event.x, event.y)
            tag_name = self.__canvas.gettags(closesttag)
            self.__clicked_checkbox = tag_name[0]

    def __checkbox_mouse_up(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)

        # if tuple tag_name is empty that means the mouse was released outside the button area
        if len(tag_name) and self.__clicked_checkbox == tag_name[0]:
            self.__toggle_checkbox()


    def __toggle_checkbox(self):
        if self.__last_status:
            self.__canvas.itemconfig(self.__image_tag_selected, state='hidden')
            self.__canvas.itemconfig(self.__image_tag_unselected, state='normal')
            self.__last_status = False
        else:
            self.__canvas.itemconfig(self.__image_tag_selected, state='normal')
            self.__canvas.itemconfig(self.__image_tag_unselected, state='hidden')
            self.__last_status = True

        if self.__command is not None:
            self.__command(f"{self.__tag}:{self.__last_status}")


    def __is_point_in_polygon(self, x, y, points):
        # Convert points to a format suitable for cv2.pointPolygonTest
        contour = np.array(points, dtype=np.int32)
        return cv2.pointPolygonTest(contour, (x, y), False) >= 0


    def __draw_polygon(self):
        self.__canvas.create_polygon(self.__polygon, fill='', outline='', tags=self.__tag, width=0)
        self.__canvas.tag_bind(self.__tag, '<Button-1>', self.__checkbox_mouse_dn)
        self.__canvas.tag_bind(self.__tag, '<ButtonRelease>', self.__checkbox_mouse_up)

    @staticmethod
    def find_control_centroid(control_poly_path, control_type, control_name):
        with open(control_poly_path, 'r') as file:
            reader = csv.reader(file, delimiter=':')
            found = False
            for row in reader:
                # print(row)
                if row[0] == control_type and row[1] == control_name:
                    result_tuple = ast.literal_eval(row[2])
                    found = True
                    # The format of centroid in the file is List of a single tuple
                    return result_tuple[0]

        if not found:
            raise ValueError(f"Control type={control_type} or Control Name={control_name} not "
                             f"found in {control_poly_path}")
        return None

    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                # print(f'str_to_coordinates - {control_name} found...')
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates



import re
import tkinter as tk
from PIL import Image, ImageTk, ImageDraw
from enum import Enum


class Draw_Over_Under(Enum):
    OVER = True
    UNDER = False

class Btn_S2:
    def __init__(self, btnName, canvas, base_image_size, polygon, base_image, color_selected=(79, 18, 9, 180),
                 drawmode=Draw_Over_Under.OVER, command=None):
        self.__tk_image = None
        self.__command = command
        self.__base_image = base_image
        self.__canvas = canvas
        self.__base_image_size = base_image_size
        self.__color_selected = color_selected
        self.__btnName = btnName
        self.__drawmode = drawmode
        self.__polygon = polygon
        self.__selected_Btn = None
        self.__draw_polygon()
        self.__saved_image = None


    def __add_remove_skin(self, status: bool):
        trans_image = Image.new('RGBA', self.__base_image_size, (255, 255, 255, 0))
        fill_color = None

        # Select color to be filled in the polygon when its created based on Valid click event
        if status:
            fill_color = self.__color_selected
        else:
            fill_color = (0, 0, 0, 0)

        def draw_polygon(fill_color):
            draw = ImageDraw.Draw(trans_image, "RGBA")
            draw.polygon(self.__polygon, fill_color)

        draw_polygon(fill_color=fill_color)

        if self.__drawmode == Draw_Over_Under.OVER:
            self.__saved_image = Image.alpha_composite(self.__base_image, trans_image)

        if self.__drawmode == Draw_Over_Under.UNDER:
            self.__saved_image = Image.alpha_composite(trans_image, self.__base_image)

        combined_image = self.__saved_image

        self.__tk_image = ImageTk.PhotoImage(combined_image)
        self.__canvas.itemconfig('baseImage', image=self.__tk_image)
        del combined_image
        del trans_image

    def __draw_polygon(self):
        tag = self.__btnName
        self.__canvas.create_polygon(self.__polygon, fill='', outline='', tags=tag, width=0)
        self.__canvas.tag_bind(tag, '<ButtonPress>', self.__top_layer_mouse_dn)
        self.__canvas.tag_bind(tag, '<B1-Motion>', self.__top_layer_mouse_drag)
        self.__canvas.tag_bind(tag, '<ButtonRelease>', self.__top_layer_mouse_up)


    def __top_layer_mouse_dn(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)
        self.__add_remove_skin(True)


    def __top_layer_mouse_drag(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)

        if tag_name[0] == self.__btnName:
            self.__add_remove_skin(True)
        else:
            self.__add_remove_skin(False)

    def __top_layer_mouse_up(self, event):
        self.__add_remove_skin(False)
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)
        self.__add_remove_skin(False)

        if self.__command is not None and tag_name[0] == self.__btnName:
            self.__command(tag_name[0])


    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates



import ast
import csv
import re
import tkinter as tk
from enum import Enum

import cv2
import numpy as np
from PIL import Image, ImageTk

from GIMPy_Widget_UI import AnalogGauge as agauge

class SliderUPPERLOWER(Enum):
    UPPER = True
    LOWER = False


class KnobS1:
    def __init__(self, root, canvas, knob_name, knob_path, polygon, knob_x_y:list, knob_range:list, max_angle,
                 resolution=0.1, x_offset_correction=0, init_angle_rotation=0, command=None):
        self.__command = command
        self.__root = root
        self.__knob_name = knob_name
        self.__canvas = canvas
        self.__polygon = polygon
        self.__tag = knob_name
        self.__knob_path = knob_path
        self.__resolution = resolution
        self.__x_loc, self.__y_loc = knob_x_y[0], knob_x_y[1]
        self.__init_angle_rotation = init_angle_rotation
        # starts with upper scale by default
        self.__scale_upper_in_use = True
        self.__x_offset_correction = x_offset_correction

        # lowerScaleVal should be the lowest value that is marked on the gauge image
        self.__lowerScaleVal = knob_range[0]
        # highScaleVal should be the highest value that is marked on the gauge image
        self.__highScaleVal = knob_range[1]

        # Copy of first list item
        self.__knob_min = knob_range[0]
        self.__knob_max = knob_range[1]
        self.__max_angle = max_angle

        # Copy of second list item
        self.__lower_slider_min = 0
        self.__lower_slider_max = 0

        # dc_amp_needle_path = 'DC_Amp_Needle.png'
        self.__knob_image = Image.open(self.__knob_path).convert("RGBA")
        self.__knob_photo = ImageTk.PhotoImage(self.__knob_image)
        self.__knob_tag_name = f'tag_{self.__knob_name}'
        canvas.create_image(self.__x_loc, self.__y_loc, anchor=tk.CENTER, image=self.__knob_photo,
                            tags=self.__knob_tag_name)

        self.__knob = agauge(root, canvas, knob_path, self.__x_loc, self.__y_loc, self.__knob_min,
                              self.__knob_max, self.__max_angle, self.__knob_tag_name,
                             init_rotation_angle=init_angle_rotation)
        # self.__knob.set_angle(self.__init_angle_rotation)

        self.__current_value = self.__knob_min
        self.__draw_polygon()


    def set(self, value: float):
        self.__current_value = value if (self.__knob_min <= value <= self.__knob_max) else self.__current_value
        self.__knob.set(self.__current_value)


    @property
    def Value(self):
        return self.__current_value

    def __is_point_in_polygon(self, x, y, points):
        """
        Function that returns True if x,y falls either in or on the list of points, False otherwise
        :param x: x coordinate
        :param y:  y coordinate
        :param points: List of coordinates
        :return: True if x,y falls either in or on the list of points, False otherwise
        """
        # Convert points to a format suitable for cv2.pointPolygonTest
        contour = np.array(points, dtype=np.int32)
        # if the point is "in" or "on" the polygon then return True
        return cv2.pointPolygonTest(contour, (x, y), False) >= 0


    def __slider_mouse_wheel(self, event):
        x, y = event.x, event.y
        mouse_wheel_direction = None
        if event.delta > 0:
            mouse_wheel_direction = 'up'
        else:
            mouse_wheel_direction = 'dn'

        if self.__is_point_in_polygon(x, y, self.__polygon):
            if mouse_wheel_direction == 'up':
                self.__current_value = self.__current_value + self.__resolution \
                    if self.__current_value + self.__resolution < self.__knob_max \
                    else self.__knob_max
            else:
                self.__current_value = self.__current_value - self.__resolution \
                    if self.__current_value - self.__resolution > self.__knob_min \
                    else self.__knob_min

            self.__current_value = round(self.__current_value / self.__resolution) * self.__resolution

            self.__knob.set(self.__current_value)

            if self.__command is not None:
                self.__command(self.__current_value)


    def __draw_polygon(self):
        self.__canvas.create_polygon(self.__polygon, fill='', outline='', tags=self.__tag, width=0)
        self.__canvas.bind('<MouseWheel>', self.__slider_mouse_wheel, add='+')


    @staticmethod
    def find_control_centroid(control_poly_path, control_type, control_name):
        found = False
        with open(control_poly_path, 'r') as file:
            reader = csv.reader(file, delimiter=':')
            for row in reader:
                # print(row)
                if row[0] == control_type and row[1] == control_name:
                    result_tuple = ast.literal_eval(row[2])
                    found = True
                    # The format of centroid in the file is List of a single tuple
                    return result_tuple[0]
        if not found:
            raise ValueError(f"Control type={control_type} or Control Name={control_name} not "
                             f"found in {control_poly_path}")
        return None

    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                # print(f'str_to_coordinates - {control_name} found...')
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates


import ast
import csv
import tkinter as tk
import re

from PIL import Image, ImageTk

class AnalogGauge:
    def __init__(self, root, canvas, needle_only_path, needle_centroid_x: int, needle_centroid_y: int,
                 lowerScaleVal: float, higherScaleVal: float, maxAngle:float, needle_tag: str, init_rotation_angle=0,
                 command=None):
        self.__root = root
        self.__canvas = canvas
        self.__command = command
        # In the gauge+needle overlayed image, if needle coincides with the lowest value then minAngle is 0.
        # Assumption here is that this will always be the case.
        self.__minAngle = 0
        self.__maxAngle = maxAngle

        self.__needle_tag = needle_tag
        # lowerScaleVal is the lowest value that is marked on the gauge image
        self.__lowerScaleVal = lowerScaleVal
        # highScaleVal is the highest value that is marked on the gauge image
        self.__highScaleVal = higherScaleVal

        self.__needle_image = Image.open(needle_only_path)
        self.__needle_image = self.__needle_image.rotate(-1 * init_rotation_angle)

        self.__image_width, self.image_height = self.__needle_image.size
        self.__needle_center_x = needle_centroid_x
        self.__needle_center_y = needle_centroid_y
        self.__tk_image = ImageTk.PhotoImage(self.__needle_image)
        # make needle visible once an instance is created
        self.__update_gauge_needle(self.__lowerScaleVal)
        self.__last_value = self.__lowerScaleVal


    def __update_gauge_needle(self, value: float):
        # Positive angles rotates the image anticlockwise and negative angle parameter goes clockwise
        # so for moving the needle we multiply with -1 after angle calculation to rotate the needle clockwise
        angle = self.__calculate_angle(value)
        rotated_image = self.__needle_image.rotate(angle, resample=Image.BICUBIC, expand=True)
        self.__tk_image = ImageTk.PhotoImage(rotated_image)
        self.__canvas.itemconfig(self.__needle_tag, image=self.__tk_image)
        self.__last_value = value

        if self.__command is not None:
            self.__command(angle)
        # self.canvas.gauge_image = self.tk_image  # Keep a reference to the image

    def __calculate_angle(self, value):
        return self.__scale_number(int(value), self.__lowerScaleVal, self.__highScaleVal, self.__minAngle, self.__maxAngle)

    def __scale_number(self, value, lowScaleVal, highscaleval, minangle, maxangle):
        scaled_value = ((int(value) - lowScaleVal) / (highscaleval - lowScaleVal)) * (maxangle - minangle) + minangle
        return scaled_value * -1

    def set(self, value):
        self.__update_gauge_needle(value)

    def set_angle(self, value):
        """
        Recommended to use this only once at the time of creating object
        :param value: angle in degrees
        :return: None
        """
        angle = -1 * value if 0 <= value <= 360 else 0
        rotated_image = self.__needle_image.rotate(angle, resample=Image.BICUBIC, expand=True)
        self.__tk_image = ImageTk.PhotoImage(rotated_image)
        self.__canvas.itemconfig(self.__needle_tag, image=self.__tk_image)
        self.__needle_image = rotated_image
        self.__last_value = self.__lowerScaleVal

    @staticmethod
    def find_control_centroid(control_poly_path, control_type, control_name):
        found = False
        with open(control_poly_path, 'r') as file:
            reader = csv.reader(file, delimiter=':')
            for row in reader:
                # print(row)
                if row[0] == control_type and row[1] == control_name:
                    result_tuple = ast.literal_eval(row[2])
                    # The format of centroid in the file is List of a single tuple
                    return result_tuple[0]
        if not found:
            raise ValueError(f"Control type={control_type} or Control Name={control_name} not "
                             f"found in {control_poly_path}")
        return None

    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                # print(f'str_to_coordinates - {control_name} found...')
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates


from .Btn_S1 import Btn_S1
from .Btn_S2 import Btn_S2, Draw_Over_Under
from .CheckBox_S1 import CheckBox_S1
from .CheckBox_S2 import CheckBox_S2
from .ON_OFF import On_Off_Button
from .Radio_S1 import RadioS1
from .AnalogGauge import AnalogGauge
from .Slider_S1 import SliderGaugeS1, SliderUPPERLOWER
from .knob_S1 import KnobS1
from .map_gimpy import map_gimpy


import tkinter as tk
from enum import Enum
import re
from PIL import Image, ImageTk, ImageDraw
import cv2
import numpy as np


class Draw_Over_Under(Enum):
    OVER = True
    UNDER = False


class RadioClassS2:
    def __init__(self, radioBtnName: str, canvas, polygon: list, imageLayers: dict, drawmode=Draw_Over_Under.OVER,
                 color_selected=(2, 255, 255, 180), default=False, command=None):
        self.__canvas = canvas
        self.__btnName = radioBtnName
        self.__drawmode = drawmode
        self.__color_selected = color_selected
        self.__command = command
        self.__imageLayers = imageLayers
        # imagelayers dictionary items consists of a list of images. Base image + an initially blank saved image
        __radio_grp_name = str(radioBtnName).split('*')[0]
        self.__base_image, self.__savedimage = self.__imageLayers["Radio_Style1*" + __radio_grp_name]

        self.__polygon = polygon
        self.__draw_polygon()
        self.__tk_image = None

        self.__last_value = None
        # This will select the default radio button without the user selecting it
        if not default:
            self.__change_radio_Status(radioBtnName, False)
            self.__last_value = False
        else:
            # print(f'{radioBtnName} is set as Default')
            self.__change_radio_Status(radioBtnName, True)
            self.__last_value = True

        self.__selected_points = None
        self.__bring_polygons_to_top()

    def __bring_polygons_to_top(self):
        # Find all items on the canvas
        items = self.__canvas.find_all()
        # Iterate through the items and bring polygons to the top
        for item in items:
            if self.__canvas.type(item) == 'polygon':
                self.__canvas.tag_raise(item)

    def __findallchildren(self):
        items = self.__canvas.find_all()
        print(f"{len(items)} items found...")

        for item in items:
            print(f"Item ID: {item}, Type: {self.__canvas.type(item)}")
            item_tags = self.__canvas.gettags(item)
            item_name = item_tags[0] if item_tags else "No name"
            print(f"Item ID: {item}, Name: {item_name}")

    def __load_image(self):
        # Load the image using PIL
        self.__tk_image = ImageTk.PhotoImage(self.pil_image)
        self.__canvas.create_image(0, 0, anchor=tk.NW, image=self.__tk_image, tags=self.__btnName + "TempRadioImage")

    def __draw_polygon(self):
        tag = self.__btnName
        self.__canvas.create_polygon(self.__polygon, fill='', outline='', tags=tag, width=0)
        self.__canvas.tag_bind(tag, '<ButtonPress>', self.__radio_top_layer_mouse_dn)
        self.__canvas.tag_bind(tag, '<ButtonRelease>', self.__radio_top_layer_mouse_up)

    def __radio_top_layer_mouse_dn(self, event):
        x, y = event.x, event.y

        if self.__is_point_in_polygon(x, y, self.__polygon):
            closesttag = self.__canvas.find_closest(event.x, event.y)
            tag_name = self.__canvas.gettags(closesttag)
            self.selected_Btn = tag_name[0]
            self.__selected_points = self.__polygon

    def __radio_top_layer_mouse_up(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)

        # if tuple tag_name is empty that means the mouse was released outside the button area
        if len(tag_name) and self.selected_Btn == tag_name[0]:
            self.__change_radio_Status(self.selected_Btn, True)

            if self.__command is not None:
                self.__command(self.selected_Btn)
                # Placeholder to call function that handles button press

        else:
            self.selected_Btn = None

        # self.findallchildren()

    def __is_point_in_polygon(self, x, y, points):
        # Convert points to a format suitable for cv2.pointPolygonTest
        contour = np.array(points, dtype=np.int32)
        return cv2.pointPolygonTest(contour, (x, y), False) >= 0

    def __change_radio_Status(self, selected_btn, Status):
        # Pass selected_btn as argument to this function so that the function can also be called
        # for a default button when creating an instance for a radio.
        trans_image = Image.new('RGBA', self.__imageLayers['PassiveLayer'][1].size, (255, 255, 255, 0))
        radio_grp_name = str(selected_btn).split('*')[0]

        # Select color to be filled in the polygon when its created based on Valid click event
        if Status:
            fill_color = self.__color_selected
        else:
            fill_color = (0, 0, 0, 0)

        def draw_polygon(fill_color):
            draw = ImageDraw.Draw(trans_image, "RGBA")
            draw.polygon(self.__polygon, fill_color)

        draw_polygon(fill_color=fill_color)

        if self.__drawmode == Draw_Over_Under.OVER:
            self.__savedimage = Image.alpha_composite(self.__base_image, trans_image)

        if self.__drawmode == Draw_Over_Under.UNDER:
            self.__savedimage = Image.alpha_composite(trans_image, self.__base_image)

        self.__imageLayers["Radio_Style1*" + radio_grp_name][1] = self.__savedimage

        combined_image = self.__imageLayers["PassiveLayer"][1]

        for key in self.__imageLayers:
            combined_image = Image.alpha_composite(combined_image, self.__imageLayers[key][1])

        self.__tk_image = ImageTk.PhotoImage(combined_image)
        self.__canvas.itemconfig('baseImage', image=self.__tk_image)
        # print("Canvas Updated")
        self.__bring_polygons_to_top()
        del combined_image
        del trans_image


    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break
        return coordinates



import re
import tkinter as tk
from PIL import Image, ImageTk
import cv2
import numpy as np


class Btn_S1:
    def __init__(self, btnName, canvas, btn_shape_points, press_drag_color = (255, 255, 255), command=None):
        # self.root = root
        self.__canvas = canvas
        self.__btnName = btnName
        # self.image_path = image_path
        self.__btn_shape_points = btn_shape_points
        self.__selected_Btn = None
        self.__command = command
        self.__press_drag_color = press_drag_color
        self.__draw_btn_shape()

    def __load_image(self):
        # Load the image using PIL
        pil_image = Image.open(self.image_path)
        self.__tk_image = ImageTk.PhotoImage(pil_image)
        self.__canvas.create_image(0, 0, anchor=tk.NW, image=self.__tk_image)

    def __draw_btn_shape(self):
        tag = self.__btnName
        # print(tag)
        self.__canvas.create_polygon(self.__btn_shape_points, fill='', outline='', tags=tag, width=0)
        self.__canvas.tag_bind(tag, '<Button-1>', self.__top_layer_mouse_dn)
        self.__canvas.tag_bind(tag, '<ButtonPress>', self.__top_layer_mouse_dn)
        self.__canvas.tag_bind(tag, '<B1-Motion>', self.__top_layer_mouse_drag)
        self.__canvas.tag_bind(tag, '<ButtonRelease>', self.__top_layer_mouse_up)


    def __rgb_to_hex(self, rgb):
        return '#{:02x}{:02x}{:02x}'.format(rgb[0], rgb[1], rgb[2])

    def __buttonpresseffect(self, btn_name, enable_effect):
        # stipple_file = "@Raw Images for Sample GUI/Layered_Groups/ClassDemoImages/Tick_Mark_XBM.xbm"
        if enable_effect:
            self.__canvas.itemconfig(btn_name, fill=self.__rgb_to_hex(self.__press_drag_color), stipple='gray25')
            # self.canvas.itemconfig(btn_name, fill=self.rgb_to_hex((230, 190, 130)), stipple=stipple_file)
        else:
            self.__canvas.itemconfig(btn_name, fill='')


    def __top_layer_mouse_dn(self, event):
        x, y = event.x, event.y

        if self.__is_point_in_polygon(x, y, self.__btn_shape_points):
            closesttag = self.__canvas.find_closest(event.x, event.y)
            tag_name = self.__canvas.gettags(closesttag)
            self.__selected_Btn = tag_name[0]
            self.__buttonpresseffect(self.__selected_Btn, True)


    def __top_layer_mouse_drag(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)

        # if tuple tag_name is empty that means the mouse is outside the button area
        if not len(tag_name):
            self.__buttonpresseffect(self.__selected_Btn, False)
        else:
            if self.__selected_Btn == tag_name[0]:
                self.__buttonpresseffect(self.__selected_Btn, True)
            else:
                self.__buttonpresseffect(self.__selected_Btn, False)

    def __top_layer_mouse_up(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)

        if len(tag_name) and self.__selected_Btn == tag_name[0]:
            if self.__command is not None:
                self.__command(self.__selected_Btn)
            # Placeholder to call function that handles button press
        else:
            self.__selected_Btn = None

        self.__buttonpresseffect(self.__selected_Btn, False)

    def __is_point_in_polygon(self, x, y, points):
        # Convert points to a format suitable for cv2.pointPolygonTest
        contour = np.array(points, dtype=np.int32)
        return cv2.pointPolygonTest(contour, (x, y), False) >= 0

    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates



import re
import tkinter as tk
from enum import Enum

import cv2
import numpy as np
from PIL import Image, ImageTk


class SliderUPPERLOWER(Enum):
    UPPER = True
    LOWER = False


class SliderGaugeS1:
    def __init__(self, root, canvas, slider_name, needle_only_paths:list, polygon, upper_slider_range:list, lower_slider_range:list,
                 dual_mode=False, resolution=0.1, x_offset_correction=0, command=None):
        self.__command = command
        self.__root = root
        self.__canvas = canvas
        self.__polygon = polygon
        self.__tag = slider_name
        self.__resolution = resolution
        self.__decimal_places = self.__count_decimal_places(self.__resolution)

        # starts with upper scale by default
        self.__scale_upper_in_use = True
        self.__dual_mode = dual_mode
        self.__x_offset_correction = x_offset_correction

        # lowerScaleVal should be the lowest value that is marked on the gauge image
        self.__lowerScaleVal = upper_slider_range[0]
        # highScaleVal should be the highest value that is marked on the gauge image
        self.__highScaleVal = upper_slider_range[1]

        # Copy of first list item
        self.__upper_slider_min = upper_slider_range[0]
        self.__upper_slider_max = upper_slider_range[1]

        # Copy of second list item
        self.__lower_slider_min = lower_slider_range[0]
        self.__lower_slider_max = lower_slider_range[1]

        self.__needle_image_upper = Image.open(needle_only_paths[0])

        # Copy of second item in the list
        self.__needle_image_lower = Image.open(needle_only_paths[1])
        self.__image_tag = 'sliderneedleImage_' + str(slider_name)
        self.__needle_image_width, self.image_height = self.__needle_image_upper.size

        self.__tk_image_upper = ImageTk.PhotoImage(self.__needle_image_upper)
        self.__tk_image_lower = ImageTk.PhotoImage(self.__needle_image_lower)

        canvas.create_image(0, 0, anchor=tk.NW, image=self.__tk_image_upper, tags=self.__image_tag)

        self.__draw_polygon()

        #
        # points = np.array(self.polygon, dtype=np.int32)
        # self.control_x_offset, self.location_y_offset, self.rect_width, self.rect_height = cv2.boundingRect(points)
        self.__control_x_offset = self.__find_min_x(self.__polygon)
        self.__rect_width = self.__find_max_x(self.__polygon) - self.__control_x_offset
        self.__location_y_offset = self.__find_min_y(self.__polygon)

        self.__units_per_pixel = (self.__highScaleVal - self.__lowerScaleVal) / self.__rect_width

        # To record x coord of last clicked position

        # make needle visible at the beginning of the scale
        self.__last_x_location = self.__control_x_offset
        self.__last_slider_value = self.__update_needle_location(self.__control_x_offset)

    def __find_min_x(self, coordinates):  # Extract the x values from the list of coordinates
        x_values = [x for x, y in coordinates]  # Find the minimum x value
        min_x = min(x_values)
        return min_x

    def __find_min_y(self, coordinates):  # Extract the x values from the list of coordinates
        y_values = [y for x, y in coordinates]  # Find the minimum x value
        min_y = min(y_values)
        return min_y

    def __find_max_x(self, coordinates):
        # Extract the x values from the list of coordinates
        x_values = [x for x, y in coordinates]
        # Find the maximum x value
        max_x = max(x_values)
        return max_x


    def set(self, slider: SliderUPPERLOWER, value: float):
        OK_to_change = False

        if (self.__scale_upper_in_use and slider == SliderUPPERLOWER.UPPER
                and (self.__upper_slider_min <= value <= self.__upper_slider_max)):
            OK_to_change = True
            new_slider_location = self.__scale_number(self.__last_slider_value, self.__lowerScaleVal, self.__highScaleVal,
                                                      self.__control_x_offset, self.__control_x_offset + self.__rect_width)
            self.__last_x_location = new_slider_location
            self.__last_slider_value = self.__update_needle_location(self.__last_x_location)

        # If dual range is enabled then change scale in use to lower before updating slider
        if (self.__dual_mode and slider == SliderUPPERLOWER.LOWER and self.__scale_upper_in_use
                and (self.__lower_slider_min <= value <= self.__lower_slider_max)):

            OK_to_change = True

            self.__scale_upper_in_use = False
            self.__highScaleVal = self.__lower_slider_max
            self.__lowerScaleVal = self.__lower_slider_min
            self.__canvas.itemconfig(self.__image_tag, image=self.__tk_image_lower)
            new_slider_location = self.__scale_number(self.__last_slider_value, self.__lowerScaleVal, self.__highScaleVal,
                                                      self.__control_x_offset, self.__control_x_offset + self.__rect_width)
            self.__last_x_location = new_slider_location
            self.__last_slider_value = self.__update_needle_location(self.__last_x_location)


        if OK_to_change:
            new_slider_location = self.__scale_number(value, self.__lowerScaleVal, self.__highScaleVal,
                                                      self.__control_x_offset, self.__control_x_offset + self.__rect_width)
            self.__last_slider_value = self.__update_needle_location(new_slider_location)
            self.__last_x_location = new_slider_location


    @property
    def Value(self):
        return self.__last_slider_value

    def __is_point_in_polygon(self, x, y, points):
        """
        Function that returns True if x,y falls either in or on the list of points, False otherwise
        :param x: x coordinate
        :param y:  y coordinate
        :param points: List of coordinates
        :return: True if x,y falls either in or on the list of points, False otherwise
        """
        # Convert points to a format suitable for cv2.pointPolygonTest
        contour = np.array(points, dtype=np.int32)
        # if the point is "in" or "on" the polygon then return True
        return cv2.pointPolygonTest(contour, (x, y), False) >= 0


    def __slider_mouse_dn(self, event):
        x, y = event.x, event.y

        if self.__is_point_in_polygon(x, y, self.__polygon):
            self.__last_slider_value = self.__update_needle_location(x)
            new_slider_value = self.__scale_number(self.__last_slider_value, self.__lowerScaleVal, self.__highScaleVal,
                                                   self.__control_x_offset, self.__control_x_offset + self.__rect_width)
            self.__last_x_location = new_slider_value

    def __slider_Right_mouse_dn(self, event):
        x, y = event.x, event.y

        if self.__dual_mode and self.__is_point_in_polygon(x, y, self.__polygon):

            if self.__scale_upper_in_use:
                self.__scale_upper_in_use = False
                self.__highScaleVal = self.__lower_slider_max
                self.__lowerScaleVal = self.__lower_slider_min
                self.__canvas.itemconfig(self.__image_tag, image=self.__tk_image_lower)
                self.__last_slider_value = self.__update_needle_location(self.__last_x_location)
                new_slider_location = self.__scale_number(self.__last_slider_value, self.__lowerScaleVal, self.__highScaleVal,
                                                          self.__control_x_offset, self.__control_x_offset + self.__rect_width)
                self.__last_x_location = new_slider_location
            else:
                self.__scale_upper_in_use = True
                self.__highScaleVal = self.__upper_slider_max
                self.__lowerScaleVal = self.__upper_slider_min
                self.__canvas.itemconfig(self.__image_tag, image=self.__tk_image_upper)
                self.__last_slider_value = self.__update_needle_location(self.__last_x_location)

                new_slider_location = self.__scale_number(self.__last_slider_value, self.__lowerScaleVal, self.__highScaleVal,
                                                          self.__control_x_offset, self.__control_x_offset + self.__rect_width)
                self.__last_x_location = new_slider_location

    def __slider_mouse_wheel(self, event):
        x, y = event.x, event.y
        mouse_wheel_direction = None
        if event.delta > 0:
            mouse_wheel_direction = 'up'
        else:
            mouse_wheel_direction = 'dn'


        if self.__is_point_in_polygon(x, y, self.__polygon):
            new_slider_value = None
            if mouse_wheel_direction == 'up':
                new_slider_value = (self.__last_slider_value + self.__resolution) \
                    if ((self.__last_slider_value + self.__resolution) < self.__highScaleVal) \
                    else self.__highScaleVal
            else:
                new_slider_value = (self.__last_slider_value - self.__resolution) \
                    if ((self.__last_slider_value - self.__resolution) > self.__lowerScaleVal) \
                    else self.__lowerScaleVal

            # new_slider_value = round(new_slider_value, self.decimal_places)
            new_slider_value = round(new_slider_value / self.__resolution) * self.__resolution

            self.__last_slider_value = new_slider_value
            new_slider_location = self.__scale_number(new_slider_value, self.__lowerScaleVal, self.__highScaleVal,
                                                      self.__control_x_offset, self.__control_x_offset + self.__rect_width)
            slider_loc_after_update = self.__update_needle_location(new_slider_location)
            self.__last_x_location = new_slider_location

    def __count_decimal_places(self, number):
        count = 0
        while number != int(number):
            number *= 10
            count += 1
        return count

    def __slider_drag_value_update(self, value):
        pass

    def __draw_polygon(self):
        self.__canvas.create_polygon(self.__polygon, fill='', outline='', tags=self.__tag, width=0)
        self.__canvas.tag_bind(self.__tag, '<Button-1>', self.__slider_mouse_dn)
        self.__canvas.bind('<MouseWheel>', self.__slider_mouse_wheel, add='+')
        self.__canvas.tag_bind(self.__tag, '<Button-3>', self.__slider_Right_mouse_dn)

    def __update_needle_location(self, x):
        self.__canvas.coords(self.__image_tag, x + self.__x_offset_correction - self.__needle_image_width // 2,
                             self.__location_y_offset)
        number_to_be_rounded = self.__scale_number(x, self.__control_x_offset, self.__control_x_offset + self.__rect_width,
                                                   self.__lowerScaleVal, self.__highScaleVal)

        return_val = round(number_to_be_rounded / self.__resolution) * self.__resolution

        if self.__command is not None:

            if self.__scale_upper_in_use:
                preceding_text = str(self.__tag) + "," + "UpperSlider,"
            else:
                preceding_text = str(self.__tag) + "," + "Lower_Slider,"

            self.__command(preceding_text + str(round(return_val, 5)))

        return return_val

    def __scale_number(self, value: float, x_min: float, x_max: float, y_min: float, y_max: float):
        m = (y_max - y_min) / (x_max - x_min)
        b = y_min - m * x_min
        return (m * value) + b


    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                # print(f'str_to_coordinates - {control_name} found...')
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates


import sys

import cv2
# import numpy as np
from tkinter import Tk, Canvas, Entry, filedialog
from PIL import Image, ImageTk
import tkinter as tk

def map_gimpy():
    # Function to handle mouse click events
    def on_click(event):
        remove_all_entry_boxes()
        root.title(f"Map controls to Python - clicked outside active area")
        # contour_label.set(f"Map controls to Python - clicked outside active area")
        x, y = event.x, event.y

        for i, contour in enumerate(contours):
            if cv2.pointPolygonTest(contour, (x, y), False) >= 0:
                # contour_label.set(f"Item Number: {i + 1}")
                root.title(f"Map controls to Python, Right click item {i+1} to map controls")
                break


    def remove_all_entry_boxes():
        enter_Instances = []
        for child in root.winfo_children():
            if isinstance(child, Entry):
                enter_Instances.append(child)
                e = str(child.winfo_name())[1:]
                print(e)

        for entry in enter_Instances:
            entry.destroy()


    def rightclick_input_box(event):

        def on_enter(e):
            global user_input
            user_input = entry.get()
            # contour_label.set(f"You entered: {user_input}")
            # print(temp_list)
            centroid = (cX, cY)
            # file_path = filedialog.asksaveasfilename(defaultextension=".txt",
            #                                          filetypes=[("Text files", "*.txt"), ("All files", "*.*")])

            file_path = control_positions_file

            if file_path:
                with open(file_path, 'a') as file:
                    file.write(str(user_input) + ":" + "[" + str(centroid) + "]:" + str(temp_list))
                    file.write('\n')
                    file.close()

            entry.destroy()

        # To make sure there is only one Entry box that is active at any given time
        remove_all_entry_boxes()

        x, y = event.x, event.y
        for i, contour in enumerate(contours):
            if cv2.pointPolygonTest(contour, (x, y), False) >= 0:
                temp_list.clear()
                for point in contour:
                    for x, y in point:
                        temp_list.append((int(x), int(y)))
                M = cv2.moments(contour)

                if M["m00"] != 0:
                    cX = int(M["m10"] / M["m00"])
                    cY = int(M["m01"] / M["m00"])
                else:
                    cX, cY = 0, 0

                entry = Entry(root)
                entry.place(x=cX, y=cY)
                # Focus on the entry widget
                entry.focus_set()
                # Bind the Enter key to store the input and close the input box
                entry.bind("<Return>", on_enter)
                break


    # Load the "...Negative" image

    image = None
    neg_file_path = filedialog.askopenfilename(title="Open Active file", filetypes=[("...Active image file", "*.png"), ("All files", "*.*")])
    if neg_file_path:
        image = cv2.imread(neg_file_path)
    else:
        sys.exit("No file selected, quitting...")

    preview_file_path = filedialog.askopenfilename(title="Load Passive image file",
                                                   filetypes=[("Passive image file", "*.png"), ("All files", "*.*")])

    if not preview_file_path:
        sys.exit("No file selected, quitting...")

    control_positions_file = filedialog.askopenfilename(title="Load control positions file",
                                                    filetypes=[("Controls Position file", "*.txt"), ("All files", "*.*")])

    if not control_positions_file:
        sys.exit("No file selected, quitting...")

    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    blurred = cv2.GaussianBlur(gray, (3, 3), 0)
    edged = cv2.Canny(blurred, 10, 190)

    # Find contours
    contours, heirarchy = cv2.findContours(edged, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    # contours = sorted(contours, key=cv2.contourArea, reverse=True)

    contourList = []
    xyPoints = []
    temp_list = []
    i = 0

    # with open('Output.txt', 'w') as file:
    #     file.write("")
    #     file.close()

    for contour in contours:
        i += 1
        # print(i)
        temp_list.clear()
        xyPoints.clear()
        for point in contour:
            for x, y in point:
                xyPoints.append(int(x))
                xyPoints.append(int(y))
                tpl = (int(x), int(y))
                temp_list.append(tpl)
            contourList.append(temp_list)
        # print(temp_list)
        # print(xyPoints)

        with open('Output.txt', 'a') as file:
            file.write("\"Item" + str(i) + "\"" + ":")
            file.write(str(temp_list))
            if i != len(contours):
                file.write(",")
            file.close()
    # contourList.clear()
    contour_points = [tuple(point[0]) for contour in contours for point in contour]
    converted_points = [(int(x), int(y)) for x, y in contour_points]
    one_dimensional_list = [coordinate for point in converted_points for coordinate in point]
    #print(contour_points)
    # Sort contours by area
    contours = sorted(contours, key=cv2.contourArea)

    # Draw contours and label them
    for i, contour in enumerate(contours):
        cv2.drawContours(image, [contour], -1, (0, 155, 250), 1)
        M = cv2.moments(contour)

        if M["m00"] != 0:
            cX = int(M["m10"] / M["m00"])
            cY = int(M["m01"] / M["m00"])
        else:
            cX, cY = 0, 0
        cv2.circle(image, (cX, cY), 2, (255, 255, 255), 2)
        # cv2.putText(image, str(i + 1), (cX - 10, cY + 15), cv2.FONT_HERSHEY_PLAIN, 1, (0, 255, 250), 1)

        # Read control position file and put a label if a control has already been defined

    # Convert the image to RGB (OpenCV uses BGR by default)
    image_rgb = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    image_pil = Image.fromarray(image_rgb)

    root = Tk()
    active_layer_image = ImageTk.PhotoImage(image_pil)
    image_pil.save("Detected_Output.png")


    top_image = Image.open(preview_file_path)
    passive_layer_image = ImageTk.PhotoImage(top_image)

    # Create a Tkinter window
    root_width, root_height = top_image.size
    root_geometry = str(root_width) + "x" + str(root_width)
    root.geometry(root_geometry)
    root.title("Map controls to Python")

    canvas = Canvas(root, width=1920, height=1080)
    canvas_image = canvas.create_image(0, 0, anchor=tk.NW, image=passive_layer_image)
    canvas.place(x=0, y=0)
    # canvas.pack()


    canvas.bind("<Button-1>", on_click)

    user_input = ""
    canvas.bind("<Button-3>", rightclick_input_box)

    # put a label where ever a currently detected centroid already exists in the position file
    with open(control_positions_file, 'r') as file:
        lines = file.readlines()
        for i, contour in enumerate(contours):
            M = cv2.moments(contour)
            if M["m00"] != 0:
                cX = int(M["m10"] / M["m00"])
                cY = int(M["m01"] / M["m00"])
                search_tuple = (cX, cY)

                for line in lines:
                    if str(search_tuple) in line:
                        btn_attributes_lst = line.split(':')
                        print(search_tuple)
                        label = tk.Label(text=str(btn_attributes_lst[0] + ':' + btn_attributes_lst[1]))
                        label.place(x=cX+10, y=cY)
        file.close()

    def selectscreens(event):
        if event.keysym == 'F1':
            print("F1")
            canvas.itemconfig(canvas_image, image=active_layer_image)
        else:
            print("F2")
            canvas.itemconfig(canvas_image, image=passive_layer_image)


    root.bind('<F1>', selectscreens)
    root.bind('<F2>', selectscreens)

    # Start the Tkinter event loop
    root.mainloop()

if __name__ == "__main__":
    map_gimpy()


import ast
import csv
import re
import tkinter as tk
from enum import Enum

import cv2
import numpy as np
from PIL import Image, ImageTk
from PIL import ImageDraw


class CheckBox_S2:
    def __init__(self, canvas, checkbox_btn_name, base_imagesize, base_image, polygon: list, checkbox_on_off_image_path: list,
                 checkbox_centroid: tuple,
                 default_state_is_on=False, x_offset_correction=0, y_offset_correction=0, command=None):
        self.__tk_image = None
        self.__command = command
        self.__canvas = canvas
        self.__polygon = polygon
        self.__base_image_size = base_imagesize
        self.__base_image = base_image
        self.__x_offset_correction = x_offset_correction
        self.__y_offset_correction = y_offset_correction

        self.__checkbox_centroid = checkbox_centroid

        self.__btnName = checkbox_btn_name
        self.__tag = checkbox_btn_name
        self.__clicked_checkbox = None

        self.__default_state_is_on = default_state_is_on
        self.__x_offset_correction = x_offset_correction

        self.__selected_image = Image.open(checkbox_on_off_image_path[0])
        self.__unselected_image = Image.open(checkbox_on_off_image_path[1])

        self.__image_tag_selected = 'checkbox_img_tag_selected_' + self.__tag
        self.__image_tag_unselected = 'checkbox_img_tag_unselected_' + self.__tag

        # self.on_off_image_width, self.on_off_image_height = self.on_off_image.size
        self.__selected_photo = ImageTk.PhotoImage(self.__selected_image)
        self.__unselected_photo = ImageTk.PhotoImage(self.__unselected_image)

        __checkbox_centroid_x, __checkbox_centroid_y = checkbox_centroid[0], checkbox_centroid[1]

        self.__canvas.create_image(__checkbox_centroid_x, __checkbox_centroid_y,
                                   anchor=tk.CENTER, image=self.__selected_photo, tags=self.__image_tag_selected)
        self.__canvas.create_image(__checkbox_centroid_x, __checkbox_centroid_y,
                                   anchor=tk.CENTER, image=self.__unselected_photo, tags=self.__image_tag_unselected)

        self.__draw_polygon()
        self.__last_status = not default_state_is_on
        self.__toggle_checkbox()


    def set(self, state: bool):
        state=not state # toggle_checkbox function toggles the input so 'state' is reversed here

        if self.__last_status == state:
            self.__last_status = state
            self.__toggle_checkbox()


    @property
    def Value(self):
        return True if self.__last_status else False


    def __checkbox_mouse_dn(self, event):
        # As thie mouse down is only triggered when it's clicked in one of the two button positions so
        # no need to check whether the click has occurred within the polygon, just toggle is required
        x, y = event.x, event.y

        if self.__is_point_in_polygon(x, y, self.__polygon):
            closesttag = self.__canvas.find_closest(event.x, event.y)
            tag_name = self.__canvas.gettags(closesttag)
            self.__clicked_checkbox = tag_name[0]

    def __checkbox_mouse_hover(self, event):
        x, y = event.x, event.y
        self.__add_remove_skin(True)

    def __checkbox_mouse_leave(self, event):
        # print("Mouse Leaves")
        self.__add_remove_skin(False)

    def __checkbox_mouse_up(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)

        # if tuple tag_name is empty that means the mouse was released outside the button area
        if len(tag_name) and self.__clicked_checkbox == tag_name[0]:
            self.__add_remove_skin(False)
            self.__toggle_checkbox()

    def __add_remove_skin(self, status: bool):
        trans_image = Image.new('RGBA', self.__base_image_size, (255, 255, 255, 0))
        fill_color = None

        # Select color to be filled in the polygon when its created based on Valid click event
        if status:
            fill_color = (0, 0, 255, 100)
        else:
            fill_color = (0, 0, 0, 0)

        def draw_polygon(fill_color):
            draw = ImageDraw.Draw(trans_image, "RGBA")
            draw.polygon(self.__polygon, fill_color)

        draw_polygon(fill_color=fill_color)

        saved_image = Image.alpha_composite(self.__base_image, trans_image)

        combined_image = saved_image

        self.__tk_image = ImageTk.PhotoImage(combined_image)
        self.__canvas.itemconfig('baseImage', image=self.__tk_image)
        del combined_image
        del trans_image


    def __toggle_checkbox(self):
        if self.__last_status:
            self.__canvas.itemconfig(self.__image_tag_selected, state='hidden')
            self.__canvas.itemconfig(self.__image_tag_unselected, state='normal')
            self.__last_status = False
        else:
            self.__canvas.itemconfig(self.__image_tag_selected, state='normal')
            self.__canvas.itemconfig(self.__image_tag_unselected, state='hidden')
            self.__last_status = True

        if self.__command is not None:
            self.__command(f"{self.__tag}:{self.__last_status}")

    def __is_point_in_polygon(self, x, y, points):
        # Convert points to a format suitable for cv2.pointPolygonTest
        contour = np.array(points, dtype=np.int32)
        return cv2.pointPolygonTest(contour, (x, y), False) >= 0


    def __draw_polygon(self):
        self.__canvas.create_polygon(self.__polygon, fill='', outline='', tags=self.__tag, width=0)
        self.__canvas.tag_bind(self.__tag, '<Button-1>', self.__checkbox_mouse_dn)
        self.__canvas.tag_bind(self.__tag, '<ButtonRelease>', self.__checkbox_mouse_up)
        self.__canvas.tag_bind(self.__tag, '<Enter>', self.__checkbox_mouse_hover)
        self.__canvas.tag_bind(self.__tag, '<Leave>', self.__checkbox_mouse_leave)


    @staticmethod
    def find_control_centroid(control_pos_path, control_type, control_name):
        with open(control_pos_path, 'r') as file:
            found = False
            reader = csv.reader(file, delimiter=':')
            for row in reader:
                # print(row)
                if row[0] == control_type and row[1] == control_name:
                    result_tuple = ast.literal_eval(row[2])
                    # The format of centroid in the file is List of a single tuple
                    return result_tuple[0]

        if not found:
            raise ValueError(f"Control type={control_type} or Control Name={control_name} not "
                             f"found in {control_pos_path}")

        return None


    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                # print(f'str_to_coordinates - {control_name} found...')
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates



import tkinter as tk
import re

from PIL import Image, ImageTk, ImageDraw
import cv2
import numpy as np


class RadioS1:
    def __init__(self, radioBtnName, canvas, polygon, default=False, stipple_color=(0, 100, 0), command=None):
        self.__canvas = canvas
        self.__btnName = radioBtnName
        self.__command = command
        self.__polygon = polygon
        if polygon == None:
            raise ValueError(f"{radioBtnName} point coordinates not found...")
        self.__stipple_color = stipple_color
        self.__draw_polygon()
        # set _value to None, this is used as a getter value
        self._value = None
        if not default:
            # self.selected_Btn = radioBtnName
            self.__change_radio_selected(radioBtnName, False)
        else:
            # print(f'{radioBtnName} is set as Default')
            # self.selected_Btn = radioBtnName
            self.__change_radio_selected(radioBtnName, True)

        self.__selected_points = None

    def __load_image(self):
        # Load the image using PIL
        self.__tk_image = ImageTk.PhotoImage(self.pil_image)
        self.__canvas.create_image(0, 0, anchor=tk.NW, image=self.__tk_image, tags="TempRadioImage")

    def __draw_polygon(self):
        tag = self.__btnName
        # print(tag)
        self.__canvas.create_polygon(self.__polygon, fill='', outline='', tags=tag, width=0)
        self.__canvas.tag_bind(tag, '<ButtonPress>', self.__radio_top_layer_mouse_dn)
        self.__canvas.tag_bind(tag, '<ButtonRelease>', self.__radio_top_layer_mouse_up)

    def __rgb_to_hex(self, rgb):
        return '#{:02x}{:02x}{:02x}'.format(rgb[0], rgb[1], rgb[2])

    def __buttonpresseffect(self, btn_name, enable_effect):
        if enable_effect:
            self.__canvas.itemconfig(btn_name, fill=self.__rgb_to_hex((230, 190, 130)), stipple='gray50')
        else:
            self.__canvas.itemconfig(btn_name, fill='')

    def __radio_top_layer_mouse_dn(self, event):
        x, y = event.x, event.y

        if self.__is_point_in_polygon(x, y, self.__polygon):
            closesttag = self.__canvas.find_closest(event.x, event.y)
            tag_name = self.__canvas.gettags(closesttag)
            self.selected_Btn = tag_name[0]
            self.__selected_points = self.__polygon
            # self.buttonpresseffect(self.selected_Btn, True)
            # print(f"Mouse Down: Clicked {tag_name} inside polygon at ({x}, {y})")

    def __radio_top_layer_mouse_drag(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)

        # if tuple tag_name is empty that means the mouse is outside the button area
        if not len(tag_name):
            self.__buttonpresseffect(self.selected_Btn, False)
        else:
            if self.selected_Btn == tag_name[0]:
                self.__buttonpresseffect(self.selected_Btn, True)
            else:
                self.__buttonpresseffect(self.selected_Btn, False)

    def __radio_top_layer_mouse_up(self, event):
        x, y = event.x, event.y
        closesttag = self.__canvas.find_closest(event.x, event.y)
        tag_name = self.__canvas.gettags(closesttag)
        # self.buttonpresseffect(self.selected_Btn, False)

        # if tuple tag_name is empty that means the mouse was released outside the button area
        if len(tag_name) and self.selected_Btn == tag_name[0]:
            # print(f"Mouse Up: Clicked inside polygon {self.selected_Btn} at ({x}, {y})")
            # print("Button selected")
            # self.select_radio(self.selected_Btn)
            self.__change_radio_selected(self.selected_Btn, True)
            if self.__command is not None:
                self.__command(self.selected_Btn)
            # Placeholder to call function that handles button press
        else:
            # print("Mouse left the button before releasing, not a valid button press")
            # print(f"Clicked outside polygon at ({x}, {y})")
            self.selected_Btn = None

        # self.findallchildren()

    def __is_point_in_polygon(self, x, y, points):
        # Convert points to a format suitable for cv2.pointPolygonTest
        contour = np.array(points, dtype=np.int32)
        return cv2.pointPolygonTest(contour, (x, y), False) >= 0

    def __change_radio_selected(self, selected_btn, new_status=False):
        # Pass selected_btn as argument to this function so that it can also be called
        # as a default button when creating an instance for a radio.
        # print(selected_btn)

        for item in self.__canvas.find_all():
            # print(self.__canvas.gettags(item))
            if (len(self.__canvas.gettags(item)) and self.__canvas.gettags(item)[0].
                    startswith(str(selected_btn).split('*')[0])):
                self.__canvas.itemconfig(item, fill='', stipple='')
        # self.bring_polygons_to_top()

        if new_status:
            self.__canvas.itemconfig(selected_btn, fill=self.__rgb_to_hex(self.__stipple_color), stipple='gray50')


    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                # print(f'str_to_coordinates - {control_name} found...')
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates



import ast
import csv
import re
import tkinter as tk
from enum import Enum

import cv2
import numpy as np
from PIL import Image, ImageTk


class On_Off_Button:
    def __init__(self, canvas, on_off_btn_name: str, on_off_polygon: list, button_only_image_path, on_off_centroids: list,
                 default_state_is_on=False, command=None):

        self.__command = command
        self.__canvas = canvas

        if len(on_off_polygon) > 2:
            raise ValueError(f"Expected 2, found {len(on_off_polygon)} items in the list for ON_OFF button={on_off_btn_name}")

        if None in on_off_polygon:
            raise ValueError(f"Point coordinates not found for ON_OFF button={on_off_btn_name}")

        self.__on_polygon = on_off_polygon[0]
        self.__off_polygon = on_off_polygon[1]

        if any(not isinstance(centroid, tuple) for centroid in on_off_centroids):
            raise ValueError(f"All centroid values in ON_OFF button={on_off_btn_name} list are expected to be tuples")

        self.__on_centroid = on_off_centroids[0]
        self.__off_centroid = on_off_centroids[1]

        self.__tag = on_off_btn_name

        self.__default_state_is_on = default_state_is_on

        self.__on_off_image_path = button_only_image_path
        self.__on_off_image = Image.open(self.__on_off_image_path)

        self.__image_tag = 'On_Off_img_' + self.__tag
        self.__on_off_image_width, self.on_off_image_height = self.__on_off_image.size
        self.__on_off_photo = ImageTk.PhotoImage(self.__on_off_image)

        # print(self.__on_centroid)

        canvas.create_image(self.__on_centroid[0] if default_state_is_on else self.__off_centroid[0],
                            self.__on_centroid[1] if default_state_is_on else self.__off_centroid[1],
                            anchor=tk.CENTER, image=self.__on_off_photo, tags=self.__image_tag)

        # print(self.__canvas.coords(self.__image_tag))

        self.__draw_polygons()


    def set(self, state: bool):
        if state==True:
            self.__canvas.coords(self.__image_tag, self.__on_centroid[0], self.__on_centroid[1])
        else:
            self.__canvas.coords(self.__image_tag, self.__off_centroid[0], self.__off_centroid[1])


    @property
    def Value(self):
        return True if self.__canvas.coords(self.__image_tag) == list(self.__on_centroid) else False

    def __ON_OFF_mouse_dn(self, event):
        # As thie mouse down is only triggered when it's clicked in one of the two button positions so
        # no need to check whether the click has occurred within the polygon, just toggle is required

        if list(self.__on_centroid) == self.__canvas.coords(self.__image_tag):
            x = self.__off_centroid[0]
            y = self.__off_centroid[1]
            self.__canvas.coords(self.__image_tag, x, y)
            if self.__command is not None:
                self.__command(f"{self.__tag}:{False}")
        else:
            x = self.__on_centroid[0]
            y = self.__on_centroid[1]
            self.__canvas.coords(self.__image_tag, x, y)

            if self.__command is not None:
                self.__command(f"{self.__tag}:{True}")


    def __draw_polygons(self):
        self.__canvas.create_polygon(self.__on_polygon, fill='', outline='', tags=self.__tag, width=0)
        self.__canvas.create_polygon(self.__off_polygon, fill='', outline='', tags=self.__tag, width=0)
        self.__canvas.tag_bind(self.__tag, '<Button-1>', self.__ON_OFF_mouse_dn)


    @staticmethod
    def find_control_centroid(control_poly_path, control_type, control_name):
        with open(control_poly_path, 'r') as file:
            reader = csv.reader(file, delimiter=':')
            found = False
            for row in reader:
                # print(row)
                if row[0] == control_type and row[1] == control_name:
                    result_tuple = ast.literal_eval(row[2])
                    found = True
                    # The format of centroid in the file is List of a single tuple
                    return result_tuple[0]

            if not found:
                raise ValueError(f"Control type={control_type} or Control Name={control_name} not "
                             f"found in {control_poly_path}")
        return None


    @staticmethod
    def str_to_coordinates(file_loc, control_name, parameter_offset):
        coordinates = None
        with open(file_loc, 'r') as file:
            lines = file.readlines()

        for line in lines:
            split_params = line.split(":")
            if split_params[1] == control_name:
                # print(f'str_to_coordinates - {control_name} found...')
                match = re.findall(r'\((\d+),\s*(\d+)\)', split_params[parameter_offset - 1].strip())
                coordinates = [tuple(map(int, coord)) for coord in match]
                # print(coordinates)
                break

        if coordinates is None:
            raise ValueError(f"Control Name={control_name} not found in {file_loc}")

        return coordinates



from setuptools import setup, find_packages
from os import path

from setuptools.config.expand import entry_points

working_directory = path.abspath(path.dirname(__file__))

with open(path.join(working_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='GIMPy_Widget_UI',
    version='0.1.0',
    # url='https://github.com/',
    author='Manish Kathuria',
    author_email='gimpywidgetui@gmail.com',
    description='Python classes supporting use of GUI created in GIMP',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=['asttokens', 'pillow', 'opencv-python', 'numpy'],
    entry_points={'console_scripts': ['map_gimpy = GIMPy_Widget_UI:map_gimpy', ], },
)